@@ -2,8 +2,7 @@
 #include "ui_mainwindow.h"
 
 #include <QDebug>
-
-#include "widgets/servicefilter.h"
+#include <datatypes/message.h>
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
@@ -13,37 +12,21 @@ MainWindow::MainWindow(QWidget *parent) :
 
     init();
 
-    this->isFriendsUpdate = false;
-    this->isAlbumsUpdate = false;
-    this->isPhotosUpdate = false;
-    this->isInboxMessagesUpdate = false;
-
     this->sm = new ServiceMgr(this);
 
-#ifdef Q_WS_MAEMO_5
-    this->sm->settings->setValue(SETTINGS_MULTIPANEL, false);
-#endif
-
-    imagesPanel = new ImageBrowserPanel(this->sm);
-    profilesPanel = new ProfileBrowserPanel(this->sm);
-    messagesPanel = new MessageBrowserPanel(this->sm);
-    msgComposer = new MessageComposer(this->sm);
+    imagesPanel = new ImageBrowserPanel(this->usedFont, this->sm, this);
+    profilesPanel = new ProfileBrowserPanel(this->usedFont, this->sm, this);
+    messagesPanel = new MessageBrowserPanel(this->usedFont, this->sm, this);
+    msgComposer = new MessageComposer(this->sm, this);
 
     ui->stackedWidget->addWidget(profilesPanel);
     ui->stackedWidget->addWidget(imagesPanel);
     ui->stackedWidget->addWidget(messagesPanel);
     ui->stackedWidget->addWidget(msgComposer);
 
-    ServiceFilter* filter = new ServiceFilter(sm);
-    ui->filterLayout->addWidget(filter);
-    ui->filterLayout->addStretch();
-
-    connect(filter, SIGNAL(stateChanged(DriverInfo*)), sm, SLOT(changeFilteredService(DriverInfo*)));
-
     connect(this->sm, SIGNAL(errorOccured(QString, QTransport::Action, bool)),
             this, SLOT(gotErrorMsg(QString, QTransport::Action, bool)));
     connect(imagesPanel, SIGNAL(openImage(Photo)), this, SLOT(openImage(Photo)));
-    connect(imagesPanel, SIGNAL(showFriendInfo(Friend)), this, SLOT(showFriendInfo(Friend)));
     connect(profilesPanel, SIGNAL(showFriendAlbums(Friend)), this,
             SLOT(showFriendAlbums(Friend)));
     connect(profilesPanel, SIGNAL(showFriendMessages(Friend)), this,
@@ -53,9 +36,15 @@ MainWindow::MainWindow(QWidget *parent) :
     connect(messagesPanel, SIGNAL(newMessage()), this, SLOT(createNewMessage()));
     connect(messagesPanel, SIGNAL(replyMessage(Message)), this,
             SLOT(replyMessage(Message)));
+    connect(messagesPanel, SIGNAL(deleteMessage(Message)), this, SLOT(messageDeleted(Message)));
+    connect(msgComposer, SIGNAL(saveMessage(Message)), this, SLOT(saveMessage(Message)));
+    connect(msgComposer, SIGNAL(messageSent()), this, SLOT(messageSent()));
     connect(messagesPanel, SIGNAL(editDraftMessage(Message)), this,
             SLOT(editMessage(Message)));
 
+    // device rotation
+    connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged()));
+
     // check that one account must be setup
     if (this->sm->getAccounts().length() == 0) {
         UIAccountSettings *settings = new UIAccountSettings(this->sm, this);
@@ -75,59 +64,65 @@ MainWindow::~MainWindow()
 void MainWindow::init()
 {
 #ifdef Q_WS_MAEMO_5
-    this->usedFont = QFont("Times", 12);
+    this->usedFont = QFont(this->font().family(), 14);
+    this->setAttribute(Qt::WA_Maemo5StackedWindow);
+    this->setAttribute(Qt::WA_Maemo5AutoOrientation, true);
+    this->setWindowFlags(this->windowFlags() | Qt::Window);
+    this->fullScreenButton = new FullScreenExitButton(this);
+    this->fullScreenButton->hide();
 #else
     this->usedFont = this->font();
 #endif
 
     actionGroup = new QActionGroup(this);
 
+    this->ui->mainToolBar->setIconSize(QSize(_MAINICONSIZE_,_MAINICONSIZE_));
+
     createActions();
     createMenus();
 
-#ifdef Q_WS_MAEMO_5
-    this->setAttribute(Qt::WA_Maemo5StackedWindow);
-    this->setWindowFlags(this->windowFlags() | Qt::Window);
-    this->fullScreenButton = new FullScreenExitButton(this);
-    this->fullScreenButton->hide();
-#endif
-
-    ui->toolBar->layout()->setSpacing(0);
-    ui->toolBar->layout()->setMargin(0);
-    ui->toolBar->addAction(friendAction);
-    ui->toolBar->addAction(msgAction);
-    ui->toolBar->addAction(albumAction);
-    ui->toolBar->addAction(moreAction);
+    ui->mainToolBar->layout()->setSpacing(0);
+    ui->mainToolBar->layout()->setMargin(0);
+    ui->mainToolBar->addAction(friendAction);
+    ui->mainToolBar->addAction(msgAction);
+    ui->mainToolBar->addAction(albumAction);
+    ui->mainToolBar->addAction(moreAction);
 }
 
 void MainWindow::createActions()
 {
+    bool isShowLabels = true;
+
+#ifdef Q_WS_MAEMO_5
+    isShowLabels = false;
+#endif
+
     // toolbar actions
-    friendAction = new QAction(tr("&Friends"), this);
+    friendAction = new QAction(isShowLabels ? tr("&Friends") : "", this);
     friendAction->setIcon(QIcon(":/res/friends.png"));
     friendAction->setShortcut(QKeySequence(Qt::ALT + Qt::Key_1));
     friendAction->setCheckable(true);
     connect(friendAction, SIGNAL(triggered()), SLOT(onFriendsTriggered()));
     actionGroup->addAction(friendAction);
 
-    msgAction = new QAction(tr("&Messages"), this);
+    msgAction = new QAction(isShowLabels ? tr("&Messages") : "", this);
     msgAction->setIcon(QIcon(":/res/messages.png"));
     msgAction->setShortcut(QKeySequence(Qt::ALT + Qt::Key_2));
     msgAction->setCheckable(true);
     connect(msgAction, SIGNAL(triggered()), SLOT(onMessagesTriggered()));
     actionGroup->addAction(msgAction);
 
-    albumAction = new QAction(tr("&Albums"), this);
+    albumAction = new QAction(isShowLabels ? tr("&Albums") : "", this);
     albumAction->setIcon(QIcon(":/res/albums.png"));
     albumAction->setShortcut(QKeySequence(Qt::ALT + Qt::Key_3));
     albumAction->setCheckable(true);
     connect(albumAction, SIGNAL(triggered()), SLOT(onAlbumsTriggered()));
     actionGroup->addAction(albumAction);
 
-    moreAction = new QAction(tr("&More"), this);
+    moreAction = new QAction(isShowLabels ? tr("&More") : "", this);
     moreAction->setIcon(QIcon(":/res/more.png"));
-    moreAction->setShortcut(QKeySequence(Qt::ALT + Qt::Key_4));
-    moreAction->setCheckable(false);
+    moreAction->setShortcut(QKeySequence(Qt::ALT + Qt::Key_0));
+    moreAction->setCheckable(true);
     connect(moreAction, SIGNAL(triggered()), SLOT(onMoreTriggered()));
     actionGroup->addAction(moreAction);
 
@@ -244,7 +239,6 @@ void MainWindow::onFriendsTriggered()
 // Messages
 void MainWindow::onMessagesTriggered()
 {
-    messagesPanel->showAllMessages();
     ui->stackedWidget->setCurrentIndex(MESSAGES_PAGE);
 }
 
@@ -257,7 +251,10 @@ void MainWindow::onAlbumsTriggered()
 // More
 void MainWindow::onMoreTriggered()
 {
-    ui->stackedWidget->setCurrentIndex(MSG_COMPOSER_PAGE);
+//    if (this->testAttribute(Qt::WA_Maemo5PortraitOrientation))
+//        this->setAttribute(Qt::WA_Maemo5PortraitOrientation, false);
+//    else
+//        this->setAttribute(Qt::WA_Maemo5PortraitOrientation, true);
 }
 
 void MainWindow::openImage(Photo photo)
@@ -291,9 +288,20 @@ void MainWindow::gotErrorMsg(QString errorMsg, QTransport::Action acc, bool isMa
 #else
 #ifdef ENABLE_NOTIFY
     NotifyNotification *notification;
+
+#if defined(NOTIFY_CHECK_VERSION)
+#    if NOTIFY_CHECK_VERSION(0, 7, 0)
+#        define NOTIFY_VERSION_MORE_7
+#    endif
+#endif
     /* Create notification */
     notification = notify_notification_new(APP_NAME,
-        errorMsg.toUtf8().data(), NULL, NULL);
+        errorMsg.toUtf8().data(), NULL
+#ifndef NOTIFY_VERSION_MORE_7
+        ,NULL
+#endif
+        );
+    
     if (notification) {
         /* Set timeout */
         notify_notification_set_timeout(notification, 3000);
@@ -312,19 +320,57 @@ void MainWindow::gotErrorMsg(QString errorMsg, QTransport::Action acc, bool isMa
     QMessageBox::warning(this, tr("Error occured"), errorMsg);
 #endif
 #endif
+}
 
-    if (acc == QTransport::getListFriendsAction) {
-        //this->isFriendsUpdate = false;
-        //buttonRefreshFriends->setIcon(QIcon::fromTheme("general_refresh", QIcon(":/res/general_refresh.png")));
-    }
-    if (acc == QTransport::getListAlbumsAction) {
-        //this->isAlbumsUpdate = false;
-        //buttonRefreshAlbums->setIcon(QIcon::fromTheme("general_refresh", QIcon(":/res/general_refresh.png")));
-    }
-    if (acc == QTransport::getListPhotosAction) {
-        //this->isPhotosUpdate = false;
-        //buttonRefreshPhotos->setIcon(QIcon::fromTheme("general_refresh", QIcon(":/res/general_refresh.png")));
+void MainWindow::showNotification(QString message)
+{
+#ifdef Q_WS_MAEMO_5
+    QDBusMessage msg = QDBusMessage::createMethodCall("org.freedesktop.Notifications", "/org/freedesktop/Notifications",
+                                                     "org.freedesktop.Notifications", "SystemNoteDialog");
+
+    QList<QVariant> args;
+    args.append(message);
+    args.append(static_cast<quint32>(0));
+    args.append("ok");
+
+    msg.setArguments(args);
+
+    QDBusConnection::systemBus().call(msg);
+#else
+#ifdef ENABLE_NOTIFY
+    NotifyNotification *notification;
+
+#if defined(NOTIFY_CHECK_VERSION)
+#    if NOTIFY_CHECK_VERSION(0, 7, 0)
+#        define NOTIFY_VERSION_MORE_7
+#    endif
+#endif
+    /* Create notification */
+    notification = notify_notification_new(APP_NAME,
+        message.toUtf8().data(), NULL
+#ifndef NOTIFY_VERSION_MORE_7
+        ,NULL
+#endif
+        );
+
+    if (notification) {
+        /* Set timeout */
+        notify_notification_set_timeout(notification, 3000);
+        /* Schedule notification for showing */
+        if (!notify_notification_show(notification, NULL))
+            qDebug("Failed to send notification");
+
+        /* Clean up the memory */
+        g_object_unref(notification);
+    } else {
+        qDebug("Failed to create notification");
+        QMessageBox::information(this, tr("Notification"), message);
     }
+#else /* if notify not supported */
+    qDebug("Failed to create notification");
+    QMessageBox::information(this, tr("Notification"), message);
+#endif
+#endif
 }
 
 void MainWindow::showFriendInfo(Friend fd)
@@ -377,6 +423,35 @@ void MainWindow::editMessage(Message msg)
     this->ui->stackedWidget->setCurrentIndex(MSG_COMPOSER_PAGE);
 }
 
+void MainWindow::saveMessage(Message)
+{
+    this->showNotification(tr("Your message was saved in Drafts."));
+    this->ui->stackedWidget->setCurrentIndex(MESSAGES_PAGE);
+}
+
+void MainWindow::messageSent()
+{
+    this->showNotification(tr("Your message was successfully sent."));
+    this->ui->stackedWidget->setCurrentIndex(MESSAGES_PAGE);
+}
+
+void MainWindow::messageDeleted(Message)
+{
+    this->showNotification(tr("Message was deleted."));
+}
+
+void MainWindow::changeEvent(QEvent *e)
+{
+    QMainWindow::changeEvent(e);
+    switch (e->type()) {
+    case QEvent::LanguageChange:
+        ui->retranslateUi(this);
+        break;
+    default:
+        break;
+    }
+}
+
 void MainWindow::closeEvent(QCloseEvent *)
 {
     emit closeDialogs();
@@ -417,3 +492,18 @@ void MainWindow::about()
 #endif
     windowHelp->show();
 }
+
+void MainWindow::orientationChanged()
+{
+    QRect geom = QApplication::desktop()->screenGeometry();
+    if (geom.width() > geom.height())
+    {
+        // landscape
+        this->addToolBar(Qt::LeftToolBarArea, ui->mainToolBar);
+    }
+    else
+    {
+        // portrait
+        this->addToolBar(Qt::BottomToolBarArea, ui->mainToolBar);
+    }
+}
